// (c) Copyright 2001-2002 iwombat.com all rights reserved
package com.iwombat.foundation.persist;

import java.util.*;
import java.sql.*;

import com.iwombat.util.Variant;


/**
 * Description: Wrapper and utility class for encapsulating db query results.
 * All Db query results are expressed as HashMaps of Variant data types, hashed 
 * by column name. 
 * This class includes lookups by column name and row name for a result set and 
 * constructs appropriate Variant types based on result set metadata.
 *
 * @author  $Author: brentp $
 * @version $Revision: 1.4 $
 * <br>
 * <dl><dt><b>Date:</b><dd>$Date: 2002/11/18 16:04:40 $</dd></dl>
 */

public class JdbcResultSet {

    private ArrayList   dbRows;      // result set rows
    private ArrayList   colNames;    // extracted from metadata
    private HashMap   colTypes;    // extracted form metadata
    private int       rowCount;    // tracks # of rows processed

    private void initValues() {

        dbRows   = null;
        colNames = null;
        colTypes = null;
        rowCount = 0;
    }

    /**
     * Constructor AppDbResults
     * Description: Create a new (empty) AppDbResults set
     *
     * @param results
     */

    public JdbcResultSet() {
        initValues();
    }

    /**
     * Constructor AppDbResults
     * Description: Create an AppDbResults class from a sql ResultSet.
     *
     * @param ResultSet from a sql query
     */

    public JdbcResultSet(ResultSet results) {
        initValues();
        try {
            setMetaData(results);
        } catch (SQLException sqe) {
            // re-init values and make result set null if exception occurs
            initValues();
        }
    }

    /**
     * Method get
     * Description: get returns the value of the given column name as an Object. For
     * integral types, their value is first wrapped in a standard Java
     * class with the corresponding data type. For arrays, an reference
     * to the array is returned.
     *
     * @param name String containing the column name to retrieve.
     *
     * @return Variant representing column data retrieved from the database.
     */

    public final Variant get(String name) {
        return get(name, 0);
    }

    /**
     * Method get
     * Description: get returns the value of the given column name as an Variant.
     *
     * @param name String containing the column name to retrieve, case insensitive.
     * @param row int index of row (0 relative) to access.
     *
     * @return Variant representing column data retrieved from the database.
     */

    public final Variant get(String name, int row) {

        Variant rValue   = null;
        Object  valueObj = null;

        try {
            Integer type   = (Integer)colTypes.get(name);           
            HashMap columns = (HashMap) dbRows.get(row);
            
            if (columns != null) {
                valueObj = columns.get(name);
            }
           

            switch (type.intValue()) {

            case java.sql.Types.BIGINT :

                // keep BIGINT as object
                return (new Variant((Object) valueObj));

            case java.sql.Types.BINARY :
                return (new Variant((byte[]) valueObj));

            case java.sql.Types.BIT :
                return (new Variant((Boolean) valueObj));

            case java.sql.Types.BLOB :
                return (new Variant((byte[]) valueObj));

            case java.sql.Types.CHAR :
                return (new Variant((Character) valueObj));

            case java.sql.Types.CLOB :

                // TODO we may run into a size restriction here
                return (new Variant((String) valueObj));

            case java.sql.Types.DECIMAL :
                return (new Variant((Float) valueObj));

            case java.sql.Types.DOUBLE :
                return (new Variant((Double) valueObj));

            case java.sql.Types.FLOAT :
                return (new Variant((Float) valueObj));

            case java.sql.Types.INTEGER :
                return (new Variant((Integer) valueObj));

            case java.sql.Types.JAVA_OBJECT :
                return (new Variant((Object) valueObj));

            case java.sql.Types.LONGVARBINARY :
                return (new Variant((byte[]) valueObj));

            case java.sql.Types.LONGVARCHAR :
                return (new Variant((String) valueObj));

            case java.sql.Types.NULL :
                return (new Variant());

            case java.sql.Types.NUMERIC :
                return (new Variant((Float) valueObj));

            case java.sql.Types.REAL :
                return (new Variant((Float) valueObj));

            case java.sql.Types.SMALLINT :
                return (new Variant((Integer) valueObj));

            case java.sql.Types.TIMESTAMP :
                return (new Variant((java.sql.Timestamp) valueObj));

            case java.sql.Types.TINYINT :
                return (new Variant((Integer) valueObj));

            case java.sql.Types.VARBINARY :
                return (new Variant((byte[]) valueObj));

            case java.sql.Types.VARCHAR :
                return (new Variant((String) valueObj));

            // unsupported Variant types
            case java.sql.Types.ARRAY :
                return (new Variant((Object) valueObj));

            case java.sql.Types.DATE :
                return (new Variant((Object) valueObj));

            case java.sql.Types.DISTINCT :
                return (new Variant((Object) valueObj));

            case java.sql.Types.STRUCT :
                return (new Variant((Object) valueObj));

            case java.sql.Types.TIME :
                return (new Variant((Object) valueObj));

            case java.sql.Types.REF :
                return (new Variant((Object) valueObj));

            case java.sql.Types.OTHER :
                return (new Variant((Object) valueObj));
            }
        } catch (Exception e) {
            return new Variant();
        }

        return new Variant();
    }

    /**
     * getRowCount returns the number of rows contained in this result.
     *
     * @return int Number of rows in this result.
     */

    public int getRowCount() {
        return rowCount;
    }

    /**
     * getRows returns an ArrayList of rows. Each row is in actuality another
     * ArrayList which contains the columns for the result set row.
     *
     * @return ArrayList representing rows retrieved. Each row is actually another
     * ArrayList, which contains the columns for that particular result set row.
     */

    public ArrayList getRows() {
        return dbRows;
    }

    /**
     * setMetaData uses a ResultSet to extract the column names, column types
     * and all rows returned. This information is kept as member data for later
     * processing using the public methods defined.
     *
     * @param results ResultSet containing the JDBC results of a query procedure.
     */

    protected void setMetaData(ResultSet results) 
    throws SQLException {

        try {
            java.sql.ResultSetMetaData mData = results.getMetaData();
            int                        count = mData.getColumnCount();

            dbRows   = new ArrayList();
            colNames = new ArrayList(count);
            colTypes = new HashMap(count);
            rowCount = 0;

            while (results.next()) {
                HashMap columns = new HashMap(count);

                for (int colIndex = 1; colIndex <= count; colIndex++) {
                    String columnName = null;
                    if (rowCount == 0) {
                        Integer keyValue   = new Integer(colIndex - 1);
                        columnName = mData.getColumnName(colIndex);

                        colNames.add(columnName);
                        colTypes.put(columnName, new Integer(mData.getColumnType(colIndex)));
                    }

                    columns.put(columnName, results.getObject(colIndex));
                }

                dbRows.add(columns);

                rowCount++;
            }
        } catch (SQLException e) {
            throw e;
        }
    }
}
