// (c) Copyright 2001-2002 iwombat.com all rights reserved
package com.iwombat.foundation.persist;

import com.iwombat.foundation.DomainObject;
import com.iwombat.foundation.Identifier;
import com.iwombat.foundation.environment.ConfigurationLocator;

import java.util.Collection;

/**
 * Utility to support generic persistence operation concepts.
 *
 * @author  $Author: brentp $
 * @version $Revision: 1.8 $
 * 
 * <br>
 * <dl><dt><b>Date:</b><dd>$Date: 2003/01/06 05:33:56 $</dd></dl>
 */

public class RdbmsDAOUtils {
    
    /** force singleton*/
    private RdbmsDAOUtils() {
    }

    /**
     * placeholder to express an Rdbms insert concept
     *
     * @param the domain object from which values will be retrieved to be inserted
     * @param the data access object to operate agains
     * @param the key of the operation to be performed
     */
    public static void executeGenericInsert(DomainObject domainObject, DAO dataAccessObject, String persistableOperationKey){
        
        executeGenericRdbmsUpdateOperation(domainObject, dataAccessObject, persistableOperationKey);
    }
    
    /**
     * placeholder to express an Rdbms delete concept
     *
     * @param the domain object from which values will be retrieved to be inserted
     * @param the data access object to operate agains
     * @param the key of the operation to be performed
     */
    public static void executeGenericDelete(DomainObject domainObject,DAO dataAccessObject, String persistableOperationKey){

        executeGenericRdbmsUpdateOperation(domainObject, dataAccessObject, persistableOperationKey);
    }

    /**
     * placeholder to express an Rdbms update concept
     *
     * @param the domain object from which values will be retrieved to be inserted
     * @param the data access object to operate agains
     * @param the key of the operation to be performed
     */
    public static void executeGenericUpdate(DomainObject domainObject, DAO dataAccessObject, String persistableOperationKey){

        executeGenericRdbmsUpdateOperation(domainObject, dataAccessObject, persistableOperationKey);
    }
    
    /**
     * Expresses a generic Rdbms operation
     *
     * @param the domain object from which values will be retrieved to be inserted
     * @param the data access object to operate agains
     * @param the key of the operation to be performed
     */
    public static void executeGenericPersistenceOperation(DomainObject domainObject, DAO dataAccessObject, String persistableOperationKey){
        
        executeGenericRdbmsUpdateOperation(domainObject, dataAccessObject, persistableOperationKey);
    }

    /**
     * Expresses the Rdbms concept of retrieving a single object/item by it's Id value
     *
     * @param the domain object from which values will be retrieved to be inserted
     * @param the id value representing the object to be retrieved
     * @param the data access object to operate agains
     * @param the key of the operation to be performed
     */
    public static DomainObject executeGenericGetById(Class domainObjectClass, 
                                                     Identifier id, 
                                                     DAO dataAccessObject, 
                                                     String persistableOperationKey){


        GetByIdParameterBean paramBean = new GetByIdParameterBean(id);
        
        
        DomainObject domainObject = executeGenericFetchItem(domainObjectClass,paramBean,  dataAccessObject,persistableOperationKey);
        
        //set the id if we get a real item back
        if (domainObject!=null) {
            domainObject.setId(id);
        }

        return domainObject;
    }
    
    /**
     * Retrieve a single DomainObject through the use of an input bean
     *
     * @param the domain object from which values will be retrieved to be inserted
     * @param the input Bean to use- the operation executor will retrieve values from this bean when preparing the query statement
     * @param the data access object to operate agains
     * @param the key of the operation to be performed
     */
    public static DomainObject executeGenericFetchItem( Class domainObjectClass,
                                                        Object inputBean,
                                                        DAO dataAccessObject,
                                                        String persistableOperationKey){
                                                        
        SqlPersistence persistenceProviderConfigInfo = (SqlPersistence)PersistenceProviderLocator.getPersistenceForDAO(dataAccessObject);

        String persistenceTechnologyKey = persistenceProviderConfigInfo.getPersistenceTechnologyKey();

        RdbmsDAOOperationConfig operationConfig = 
              RdbmsDAOOperationConfigLocator.getOperationConfig(dataAccessObject,persistableOperationKey, persistenceTechnologyKey);
        
        //get the active transaction token, if exists
        TransactionToken transactionToken=getTransactionToken(dataAccessObject);

        return RdbmsDAOOperationExecutor.executeFetchItemOperation( inputBean,
                                                                     domainObjectClass,
                                                                     operationConfig,
                                                                     persistenceProviderConfigInfo);

    }
    
    
    /**
     * This method prepares the objects that will be used to execute a FetchList operation.
     * 
     * @param input parameters for operation
     * @param the domain object interface to be used when creating items within the list
     * @param persistableOperationKey specifies operationKey used to retrieve prepared statement
     * @return the list of items resulting from this operation
     */
    public static Collection executeGenericFetchList(   Object inputBean, 
                                                         Class domainObjectInterfaceClass, 
                                                        String persistableOperationKey){
        DAO dataAccessObject = DAOFactory.createDAOInstance(domainObjectInterfaceClass);
	return executeGenericFetchList(inputBean, domainObjectInterfaceClass, dataAccessObject, persistableOperationKey);
    }
    /**
     * This method prepares the objects that will be used to execute a FetchList operation.
     * 
     * @param input parameters for operation
     * @param the domain object class to be used when creating items within the list. If this represents an interface the DomainObjectFactory is used to create and instance. Otherwise Class.newInstance() is used.
     * @param the DAO associated with this operation
     * @param persistableOperationKey specifies operationKey used to retrieve prepared statement
     * @return the list of items resulting from this operation
     */
    public static Collection executeGenericFetchList(   Object inputBean, 
                                                         Class domainObjectClass,
                                                           DAO dataAccessObject, 
                                                        String persistableOperationKey){

        
        SqlPersistence persistenceProviderConfigInfo = (SqlPersistence)PersistenceProviderLocator.getPersistenceForDAO(dataAccessObject);

        
        String persistenceTechnologyKey = persistenceProviderConfigInfo.getPersistenceTechnologyKey();
        
        RdbmsDAOOperationConfig operationConfig = 
                RdbmsDAOOperationConfigLocator.getOperationConfig(dataAccessObject, 
                                                                  persistableOperationKey, 
                                                                  persistenceTechnologyKey);
        
        //get the active transaction token, if exists
        // fetches shouldn't belong in transactions
        //TransactionToken transactionToken= getTransactionToken(domainObjectInterfaceClass);

        Collection list = RdbmsDAOOperationExecutor.executeFetchListOperation(inputBean ,
                                                            domainObjectClass ,
                                                            operationConfig, 
                                                            persistenceProviderConfigInfo,
                                                            null);
        
        return list;
    }
    
    
    private static void executeGenericRdbmsUpdateOperation(DomainObject domainObject,
                                                                    DAO dataAccessObject,
                                                                 String persistableOperationKey){

        SqlPersistence persistenceProviderConfigInfo =(SqlPersistence)PersistenceProviderLocator.getPersistenceForDAO(dataAccessObject);
        
        String persistenceTechnologyKey = 
                      persistenceProviderConfigInfo.getPersistenceTechnologyKey();
        
        RdbmsDAOOperationConfig operationConfig = 
                      RdbmsDAOOperationConfigLocator.getOperationConfig(dataAccessObject, 
                                                                        persistableOperationKey, persistenceTechnologyKey);
        
        //get the active transaction token, if exists
        TransactionToken transactionToken= getTransactionToken(dataAccessObject);

        RdbmsDAOOperationExecutor.executeUpdateOperation( domainObject, 
                                                          operationConfig, 
                                                          persistenceProviderConfigInfo,
                                                          transactionToken);
    }
    

    private static TransactionToken getTransactionToken(DAO dataAccessObject){
     
        TransactionToken transactionToken=null;
        if (TransactionalDAO.class.isAssignableFrom(dataAccessObject.getClass())){
            
            transactionToken=((TransactionalDAO)dataAccessObject).getTransactionToken();
        }
        return transactionToken;
    }
}
