
// (c) Copyright 2001-2002 iwombat.com all rights reserved


package com.iwombat.util;



import java.io.Serializable;



/**
 * The Enums class provides a Java answer to the enum capability found in
 * C/C++. 
 * <p>
 * Methods of using Enums follow:
 *
 * <p>The first method would be to declare specific instances of an Enums
 * derived class and use reference to this instance as the enum value. For
 * example:
 * <blockquote><pre>
 * import com.iwombat.util.*;                  // Enums
 * <br>
 * public final class ExampleEnum extends Enums
 * {
 *     static public final ExampleEnum VALUE1 = new ExampleEnum();
 *     static public final ExampleEnum VALUE2 = new ExampleEnum();
 *     static public final ExampleEnum VALUE3 = new ExampleEnum();
 * <br>
 *     private ExampleEnum()
 *     {
 *         super( 0 );
 *     }
 * <br>
 *     protected int[] getValues()
 *     {
 *         return null;
 *     }
 * }
 * </pre></blockquote>
 * <p>The second method of Enum use consists of numeric values associated with
 * each enum instance. An example implementation for this type of derivation
 * is as follows:
 * <blockquote><pre>
 * import com.iwombat.util.*;                  // Enums
 * <br>
 * public final class ExampleEnum extends Enums
 * {
 *     static private final int m_enums[] = { 1, 2, 3 };
 * <br>
 *     static public final ExampleEnum VALUE1 = new ExampleEnum( m_enums[ 0 ] );
 *     static public final ExampleEnum VALUE2 = new ExampleEnum( m_enums[ 1 ] );
 *     static public final ExampleEnum VALUE3 = new ExampleEnum( m_enums[ 2 ] );
 * <br>
 *     private ExampleEnum( int enum )
 *     {
 *         super( enum );
 *     }
 * <br>
 *     protected int[] getValues()
 *     {
 *         return m_enums;
 *     }
 * }
 * </pre></blockquote>
 *  * <p>
 * A final example in using Enums emulates the C/C++ enum statement as much as
 * possible in Java. This example allows using switch statements in a symbolic
 * style, yet it still provides the type safety features in interfaces.
 * <blockquote><pre>
 * import com.iwombat.util.*;                  // Enums
 * <br>
 * public final class ExampleEnum extends Enums
 * {
 *     static private final int M_VALUE1 = 1;
 *     static private final int M_VALUE2 = 2;
 *     static private final int M_VALUE3 = 3;
 * <br>
 *     static private final int m_enums[] = { M_VALUE1, M_VALUE2, M_VALUE3 };
 * <br>
 *     static public final ExampleEnum VALUE1 = new ExampleEnum( M_VALUE1 );
 *     static public final ExampleEnum VALUE2 = new ExampleEnum( M_VALUE2 );
 *     static public final ExampleEnum VALUE3 = new ExampleEnum( M_VALUE3 );
 * <br>
 *     private ExampleEnum( int enum )
 *     {
 *         super( enum );
 *     }
 * <br>
 *     protected int[] getValues()
 *     {
 *         return m_enums;
 *     }
 * }
 * </pre></blockquote>
 * <br>
 * a snippet of code can now be written as:
 * <br>
 * <blockquote><pre>
 * public void exampleMethod( ExampleEnum errorCode )
 * {
 *     switch ( errorCode.switchValue() )
 *     {
 *         case ExampleEnum.M_VALUE1 :
 *         case ExampleEnum.M_VALUE2 :
 *         case ExampleEnum.M_VALUE3 :
 *             break;                   // if all values are tested then the
 *         default :                    // default statement will never execute
 *             break;                   // as bogus values can't be passed in.
 *     }
 * }
 * </pre></blockquote>
 *
 *
 * @author  $Author: bob $
 * @version $Revision: 1.4 $
 * 
 * <br>
 * <dl><dt><b>Date:</b><dd>$Date: 2002/10/01 17:54:16 $</dd></dl>
 */

public abstract class Enums implements Serializable {


    private static final long serialVersionUID = 7893969451180184102L;

    /**
     * Holder for the value of this enum.
     */

    protected int m_enum;    // current value

    /**
     * Protected constructor for subclasses. Since Enums must be derived to be
     * used, implementors of these subclasses are bound to call this with the
     * integer value of each enumerator. Derived classes should have a private
     * constructor to keep instances of the class only those declared by the
     * class itself.
     *
     * @param enum integer value of this enum object.
     */

    protected Enums(int enum) {
        m_enum = enum;
    }

    /**
     * Return the number of values the derived enumerator contains.
     *
     * @return int count of enumerated values.
     */

    public final int count() {

        int[] values = getValues();

        return values.length;
    }

    /**
     * Return the integer value of an Enums object.
     *
     * @return int value of this Enums instance.
     */

    public final int getValue() {
        return m_enum;
    }

    /**
     * Required method for derived classes, getValues() returns the internal
     * array each derived class contains for their enum values. If a derived
     * class does not wish to implement the value array, it can return null
     * and the isEnumValue() method will return false as a result.
     *
     * @return array of integers containing the enum values or null.
     */

    protected abstract int[] getValues();

    /**
     * Test if a numeric value is one of the values for a given Enums object.
     *
     * @param enum integer value to test.
     *
     * @return true if enum value matches one of the values for this Enums
     * object. false if no match is found.
     */

    public final boolean isEnumValue(int enum) {

        int[] values = getValues();

        if (values == null) {
            return false;
        }

        for (int iter = 0; iter < values.length; iter++) {
            if (values[iter] == enum) {
                return true;
            }
        }

        return false;
    }
}
