// (c) 2001-2002 iWombat.com All rights reserved

package com.iwombat.util;


import junit.framework.*;
import java.sql.Timestamp;
import java.util.Vector;


/**
 * Test some rudimentary routines in the Variant class for all the supported types
 * of variants. This test performs the following checks for each supported type:
 * <ul>
 * <li> creation
 * <li> reassignment
 * <li> exception handling <code>VariantIllegalTypeException</code> for invalid forms
 *  of accessors and mutators once the underlying type has been established.
 *
 * @author  $Author: bob $
 * @version $Revision: 1.4 $
 * @date    $Date: 2002/03/24 06:37:48 $
 */

public class VariantTest extends TestCase {

    /** Creates a new instance of VariantTest */
    public VariantTest(String name) {
        super(name);
    }

    public static void main(java.lang.String[] args) {
        junit.textui.TestRunner.run(suite());
    }
    public void setUp() {

    }
        
    public static Test suite() {
	return new TestSuite(VariantTest.class);
    }
    
    /**
     * Tests the variant class for Boolean types
     * @author  bobc
     */
    
    public static void testBoolean() {
        Variant vbool  = new Variant((boolean)true);
        Variant vBoolean = new Variant(new Boolean(true));
        
        assertTrue("Variant boolean not given boolean type", vbool.getVariantType().equals(VariantEnum.BOOLEAN) );
        
        assertTrue("primitive boolean not set properly", vbool.getboolean());
        assertTrue("Object Boolean not set properly", vBoolean.getBoolean().booleanValue()  );
        try {
            long l = vbool.getlong();
            assertTrue("Illegal get of boolean to long did not throw VariantIllegalTypeException", false);
        } catch (VariantIllegalTypeException e) {
            
        }
        
        try {
            vbool.set(0.1f);
            assertTrue("Illegal set of boolean to float did not throw VariantIllegalTypeException", false);
        } catch (VariantIllegalTypeException e) {
            
        }
        
        try {
            vbool.setfloat(0.1f);
            assertTrue("Illegal set of boolean to float did not throw VariantIllegalTypeException", false);
        } catch (VariantIllegalTypeException e) {
            
        }
        
        vbool.set(false);
        assertTrue("primitive boolean not set properly", !vbool.getboolean());
        
        vbool.setboolean(true);
        assertTrue("primitive boolean not set properly", vbool.getboolean());
        
        
    }

    
    /**
     * Tests the variant class for Byte types
     * @author  bobc
     */
    public static void testByte() {
        byte b = (byte)0xff;
        byte b2 = (byte)0xa1;
        Variant var  = new Variant(b);
        Variant varObj = new Variant(new Byte(b));
        
        assertTrue("Variant byte primitive not given byte type", var.getVariantType().equals(VariantEnum.BYTE) );
        assertTrue("Variant Byte Object not given byte type", varObj.getVariantType().equals(VariantEnum.BYTE) );
        
        assertTrue("primitive byte not set properly", (b == var.getbyte()));
        assertTrue("Object Byte not set properly", (b == varObj.getByte().byteValue())  );
        try {
            boolean bool = var.getboolean();
            assertTrue("Illegal get of byte to long did not throw VariantIllegalTypeException", false);
        } catch (VariantIllegalTypeException e) {
            
        }
        
        try {
            var.set(true);
            assertTrue("Illegal set of byte to boolean did not throw VariantIllegalTypeException", false);
        } catch (VariantIllegalTypeException e) {
            
        }
        
        try {
            var.setboolean(true);
            assertTrue("Illegal set of byte to boolean did not throw VariantIllegalTypeException", false);
        } catch (VariantIllegalTypeException e) {
            
        }
        
        var.set(b2);
        assertTrue("primitive boolean not set properly", (b2 == var.getbyte()) );
        
    }
    
    
    /**
     * Tests the variant class for byte-array types
     * @author  bobc
     */
    public static void testByteArray() {
        byte[] b = HexStringUtil.bytesFromHexString("aa112277ff");
        byte[] b2 = HexStringUtil.bytesFromHexString("aa112277ff");
        
        
        
        
        VariantEnum thisType = VariantEnum.BYTE_ARRAY; 
       
        
        Variant var  = new Variant(b);
        Variant var2 = new Variant(b2);
        String myLabel = VariantEnum.typeName(thisType.getValue());
        
        assertTrue("Variant " + myLabel + "primitive not given correct type", 
                     var.getVariantType().equals(thisType) );
        
        
        assertTrue("primitive " + myLabel + " not set properly", ByteArrayUtil.sameValue(b, var.getByteArray()) );
        
        try {
            Object o = var.getObject();
            assertTrue("Illegal get of " + myLabel + " to Object did not throw VariantIllegalTypeException", false);
        } catch (VariantIllegalTypeException e) {
            
        }
        
        try {
            var.set((Object)(new Integer(12)));
            assertTrue("Illegal set of " + myLabel + " to Object did not throw VariantIllegalTypeException", false);
        } catch (VariantIllegalTypeException e) {
            
        }
        
        try {
            var.setObject((Object)(new Integer(12)));
            assertTrue("Illegal set of "  + myLabel + " to Object did not throw VariantIllegalTypeException", false);
        } catch (VariantIllegalTypeException e) {
            
        }
        
        var.set(b2);
        assertTrue("primitive " + myLabel + " not set properly", ByteArrayUtil.sameValue(b2, var.getByteArray()) );
        
    }
    
    
    /**
     * Tests the variant class for Char types
     * @author  bobc
     */
    
    public static void testChar() {
        char c = 'c';
        char c2 = 'a';
        Character cobj = new Character('a');
        
        Variant var  = new Variant(c);
        Variant varObj = new Variant(cobj);
        
        assertTrue("Variant byte primitive not given byte type", var.getVariantType().equals(VariantEnum.CHAR) );
        assertTrue("Variant Byte Object not given byte type", varObj.getVariantType().equals(VariantEnum.CHAR) );
        
        assertTrue("primitive char not set properly", (c == var.getchar()));
        assertTrue("Object Character not set properly", (cobj.equals(varObj.getCharacter()))  );
        try {
            byte b = var.getbyte();
            assertTrue("Illegal get of char to byte did not throw VariantIllegalTypeException", false);
        } catch (VariantIllegalTypeException e) {
            
        }
        
        try {
            var.set((byte)0xff);
            assertTrue("Illegal set of char to byte did not throw VariantIllegalTypeException", false);
        } catch (VariantIllegalTypeException e) {
            
        }
        
        try {
            var.setbyte((byte)0xff);
            assertTrue("Illegal set of Character to byte did not throw VariantIllegalTypeException", false);
        } catch (VariantIllegalTypeException e) {
            
        }
        
        var.set(c2);
        assertTrue("primitive character not set properly", (c2 == var.getchar()) );
        
    }
    
    public static void testDouble() {
        double t = 1.9234562D;
        double t2 = 0.5555555555555555555555555555555D;
        Double tobj = new Double(t);
        
        Variant var  = new Variant(t);
        Variant varObj = new Variant(tobj);
        
        assertTrue("Variant double primitive not given double type", var.getVariantType().equals(VariantEnum.DOUBLE) );
        assertTrue("Variant Double Object not given double type", varObj.getVariantType().equals(VariantEnum.DOUBLE) );
        
        assertTrue("primitive double not set properly", (t == var.getdouble()));
        assertTrue("Object Double not set properly", (tobj.equals(varObj.getDouble()))  );
        try {
            char c = var.getchar();
            assertTrue("Illegal get of double to char did not throw VariantIllegalTypeException", false);
        } catch (VariantIllegalTypeException e) {
            
        }
        
        try {
            var.set((char)'a');
            assertTrue("Illegal set of double to char did not throw VariantIllegalTypeException", false);
        } catch (VariantIllegalTypeException e) {
            
        }
        
        try {
            var.setchar((char)'a');
            assertTrue("Illegal set of double to char did not throw VariantIllegalTypeException", false);
        } catch (VariantIllegalTypeException e) {
            
        }
        
        var.set(t2);
        assertTrue("primitive double not set properly", (t2 == var.getdouble()) );
        
    }
    
    public static void testFloat() {
        float t = 1.9234562F;
        float t2 = 0.555555F;
        Float tobj = new Float(t);
        
        
        VariantEnum thisType = VariantEnum.FLOAT; 
       
        
        Variant var  = new Variant(t);
        Variant varObj = new Variant(tobj);
        String myLabel = VariantEnum.typeName(thisType.getValue());
        
        assertTrue("Variant " + myLabel + "primitive not given correct type", 
                     var.getVariantType().equals(thisType) );
        assertTrue("Variant " + myLabel + "Object not given correct type", 
                     varObj.getVariantType().equals(thisType) );
        
        assertTrue("primitive " + myLabel + " not set properly", (t == var.getfloat()));
        assertTrue("Object " + myLabel + " not set properly", (tobj.equals(varObj.getFloat()))  );
        try {
            double d = var.getdouble();
            assertTrue("Illegal get of " + myLabel + " to double did not throw VariantIllegalTypeException", false);
        } catch (VariantIllegalTypeException e) {
            
        }
        
        try {
            var.set((double)0.5555555555555555555555555555D);
            assertTrue("Illegal set of " + myLabel + " to char did not throw VariantIllegalTypeException", false);
        } catch (VariantIllegalTypeException e) {
            
        }
        
        try {
            var.setdouble((double)0.5555555555555555555555555D);
            assertTrue("Illegal set of "  + myLabel + " to double did not throw VariantIllegalTypeException", false);
        } catch (VariantIllegalTypeException e) {
            
        }
        
        var.set(t2);
        assertTrue("primitive " + myLabel + " not set properly", (t2 == var.getfloat()) );
        
    }
    
    public static void testInt() {
        int t = 12;
        int t2 = 32767;
        Integer tobj = new Integer(t);
        
        
        VariantEnum thisType = VariantEnum.INT; 
       
        
        Variant var  = new Variant(t);
        Variant varObj = new Variant(tobj);
        String myLabel = VariantEnum.typeName(thisType.getValue());
        
        assertTrue("Variant " + myLabel + "primitive not given correct type", 
                     var.getVariantType().equals(thisType) );
        assertTrue("Variant " + myLabel + "Object not given correct type", 
                     varObj.getVariantType().equals(thisType) );
        
        assertTrue("primitive " + myLabel + " not set properly", (t == var.getint()));
        assertTrue("Object " + myLabel + " not set properly", (tobj.equals(varObj.getInteger()))  );
        try {
            float f = var.getfloat();
            assertTrue("Illegal get of " + myLabel + " to float did not throw VariantIllegalTypeException", false);
        } catch (VariantIllegalTypeException e) {
            
        }
        
        try {
            var.set((float)0.55555F);
            assertTrue("Illegal set of " + myLabel + " to float did not throw VariantIllegalTypeException", false);
        } catch (VariantIllegalTypeException e) {
            
        }
        
        try {
            var.setdouble((double)0.5555555555555555555555555D);
            assertTrue("Illegal set of "  + myLabel + " to double did not throw VariantIllegalTypeException", false);
        } catch (VariantIllegalTypeException e) {
            
        }
        
        var.set(t2);
        assertTrue("primitive " + myLabel + " not set properly", (t2 == var.getint()) );
        
    }
    
    public static void testTimestamp() {

        long milis = System.currentTimeMillis();
        Timestamp tobj = new Timestamp((long)milis);
        Timestamp tobj2 = new Timestamp((long)milis+1);
        
        
        VariantEnum thisType = VariantEnum.SQL_TIMESTAMP; 
       
        
        Variant var  = new Variant(tobj2);
        Variant varObj = new Variant(tobj);
        String myLabel = VariantEnum.typeName(thisType.getValue());
        
        assertTrue("Variant " + myLabel + "Object not given correct type", 
                     varObj.getVariantType().equals(thisType) );
        
        assertTrue("Object " + myLabel + " not set properly", (tobj.getTime() == varObj.getJavaSqlTimestamp().getTime()));
        try {
            int i = var.getint();
            assertTrue("Illegal get of " + myLabel + " to int did not throw VariantIllegalTypeException", false);
        } catch (VariantIllegalTypeException e) {
            
        }
        
        try {
            var.set((int)12);
            assertTrue("Illegal set of " + myLabel + " to int did not throw VariantIllegalTypeException", false);
        } catch (VariantIllegalTypeException e) {
            
        }
        
        try {
            var.setint((int)13);
            assertTrue("Illegal set of "  + myLabel + " to int did not throw VariantIllegalTypeException", false);
        } catch (VariantIllegalTypeException e) {
            
        }
        
        var.set(tobj2);
        assertTrue("Object " + myLabel + " not set properly", (tobj2.getTime() == var.getJavaSqlTimestamp().getTime()));
        
    }
    
    
     public static void testLong() {
        long t = 12L;
        long t2 = 3967391023L;
        Long tobj = new Long(t);
        
        
        VariantEnum thisType = VariantEnum.LONG; 
       
        
        Variant var  = new Variant(t);
        Variant varObj = new Variant(tobj);
        String myLabel = VariantEnum.typeName(thisType.getValue());
        
        assertTrue("Variant " + myLabel + "primitive not given correct type", 
                     var.getVariantType().equals(thisType) );
        
        assertTrue("Variant " + myLabel + "Object not given correct type", 
                     varObj.getVariantType().equals(thisType) );
        
        assertTrue("primitive " + myLabel + " not set properly", (t == var.getlong()));
        assertTrue("Object " + myLabel + " not set properly", (tobj.equals(varObj.getLong()))  );
        try {
            float f = var.getfloat();
            assertTrue("Illegal get of " + myLabel + " to float did not throw VariantIllegalTypeException", false);
        } catch (VariantIllegalTypeException e) {
            
        }
        
        try {
            var.set((float)0.55555F);
            assertTrue("Illegal set of " + myLabel + " to float did not throw VariantIllegalTypeException", false);
        } catch (VariantIllegalTypeException e) {
            
        }
        
        try {
            var.setdouble((double)0.5555555555555555555555555D);
            assertTrue("Illegal set of "  + myLabel + " to double did not throw VariantIllegalTypeException", false);
        } catch (VariantIllegalTypeException e) {
            
        }
        
        var.set(t2);
        assertTrue("primitive " + myLabel + " not set properly", (t2 == var.getlong()) );
        
     
    }
    
    
     
     public static void testObject() {
       
       
        Vector tobj = new Vector(12);
        Object myObj = new Integer(16);
        
        
        VariantEnum thisType = VariantEnum.OBJECT; 
       
        Variant varObj = new Variant((Object)tobj);
        String myLabel = VariantEnum.typeName(thisType.getValue());
        
        assertTrue("Variant " + myLabel + "Object not given correct type", 
                     varObj.getVariantType().equals(thisType) );
        
        assertTrue("Object " + myLabel + " not set properly", (tobj.equals(varObj.getObject()))  );
        try {
            long l = varObj.getlong();
            assertTrue("Illegal get of " + myLabel + " to long did not throw VariantIllegalTypeException", false);
        } catch (VariantIllegalTypeException e) {
            
        }
        
        try {
            varObj.set((long)12L);
            assertTrue("Illegal set of " + myLabel + " to long did not throw VariantIllegalTypeException", false);
        } catch (VariantIllegalTypeException e) {
            
        }
        
        try {
            varObj.setlong((long)1234567890L);
            assertTrue("Illegal set of "  + myLabel + " to long did not throw VariantIllegalTypeException", false);
        } catch (VariantIllegalTypeException e) {
            
        }
        
        varObj.set(myObj);
        assertTrue("Object " + myLabel + " not set properly", (myObj.equals(varObj.getObject())) );
        
    }
     
    
    public static void testShort() {
        short t = 12;
        short t2 = 254;
        Short tobj = new Short(t);
        
        
        VariantEnum thisType = VariantEnum.SHORT; 
       
        
        Variant var  = new Variant(t);
        Variant varObj = new Variant(tobj);
        String myLabel = VariantEnum.typeName(thisType.getValue());
        
        assertTrue("Variant " + myLabel + "primitive not given correct type", 
                     var.getVariantType().equals(thisType) );
        assertTrue("Variant " + myLabel + "Object not given correct type", 
                     varObj.getVariantType().equals(thisType) );
        
        assertTrue("primitive " + myLabel + " not set properly", (t == var.getshort()));
        assertTrue("Object " + myLabel + " not set properly", (tobj.equals(varObj.getShort()))  );
        try {
            Object o = var.getObject();
            assertTrue("Illegal get of " + myLabel + " to Object did not throw VariantIllegalTypeException", false);
        } catch (VariantIllegalTypeException e) {
            
        }
        
        try {
            var.set((Object)(new Integer(12)));
            assertTrue("Illegal set of " + myLabel + " to Object did not throw VariantIllegalTypeException", false);
        } catch (VariantIllegalTypeException e) {
            
        }
        
        try {
            var.setObject((Object)(new Integer(12)));
            assertTrue("Illegal set of "  + myLabel + " to Object did not throw VariantIllegalTypeException", false);
        } catch (VariantIllegalTypeException e) {
            
        }
        
        var.set(t2);
        assertTrue("primitive " + myLabel + " not set properly", (t2 == var.getshort()) );
        
    }
    
    public static void testString() {
       
       
        String tobj = new String("write test first");
        String tobj2 = new String("optimize last");
        
        
        VariantEnum thisType = VariantEnum.STRING;
       
        Variant varObj = new Variant(tobj);
        String myLabel = VariantEnum.typeName(thisType.getValue());
        
        assertTrue("Variant " + myLabel + "Object not given correct type", 
                     varObj.getVariantType().equals(thisType) );
        
        assertTrue("Object " + myLabel + " not set properly", (tobj.equals(varObj.getString())) );
        try {
            short s = varObj.getshort();
            assertTrue("Illegal get of " + myLabel + " to short did not throw VariantIllegalTypeException", false);
        } catch (VariantIllegalTypeException e) {
            
        }
        
        try {
            varObj.set((short)12);
            assertTrue("Illegal set of " + myLabel + " to short did not throw VariantIllegalTypeException", false);
        } catch (VariantIllegalTypeException e) {
            
        }
        
        try {
            varObj.setshort((short)1234);
            assertTrue("Illegal set of "  + myLabel + " to short did not throw VariantIllegalTypeException", false);
        } catch (VariantIllegalTypeException e) {
            
        }
        
        varObj.set(tobj2);
        assertTrue("Object " + myLabel + " not set properly", (tobj2.equals(varObj.getString())) );
        
    }
    
}
